#
# installclass.py:  This is the prototypical class for workstation, server, and
# kickstart installs.  The interface to BaseInstallClass is *public* --
# ISVs/OEMs can customize the install by creating a new derived type of this
# class.
#
# Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
# Red Hat, Inc.  All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from distutils.sysconfig import get_python_lib
import os, sys
import imputil

from blivet import util
from blivet.partspec import PartSpec
from blivet.devicelibs import swap
from blivet.platform import platform
from blivet.size import Size

import logging
log = logging.getLogger("anaconda")

from pyanaconda.kickstart import getAvailableDiskSpace

class BaseInstallClass(object):
    # default to not being hidden
    hidden = False
    name = "base"
    bootloaderTimeoutDefault = None
    bootloaderExtraArgs = []

    # Anaconda flags several packages to be installed based on the configuration
    # of the system -- things like fs utilities, bootloader, &c. This is a list
    # of packages that we should not try to install using the aforementioned
    # mechanism.
    ignoredPackages = []

    # This flag controls whether or not Anaconda should provide an option to
    # install the latest updates during installation source selection.
    installUpdates = True

    _l10n_domain = None

    # The default filesystem type to use.  If None, we will use whatever
    # Blivet uses by default.
    defaultFS = None

    # help
    help_folder = "/usr/share/anaconda/help"
    help_main_page = "Installation_Guide.xml"
    help_placeholder = None
    help_placeholder_with_links = None

    @property
    def l10n_domain(self):
        if self._l10n_domain is None:
            raise RuntimeError("Localization domain for '%s' not set." %
                               self.name)
        return self._l10n_domain

    def setPackageSelection(self, anaconda):
        pass

    def getBackend(self):
        # The default is to return None here, which means anaconda should
        # use live or yum (whichever can be detected).  This method is
        # provided as a way for other products to specify their own.
        return None

    def setDefaultPartitioning(self, storage):
	log.info("in function setDefaultPartitioning")
	#res = os.system("cat /proc/cmdline | grep PMON_BIOS") #bootfstype
	#if res == 0:
	#    filesystem = "ext3"
	#else:
	#    filesystem = storage.defaultFSType
	res = os.system("ls /tmp/.overlay 2>/dev/null")
	if res == 0:
	    OVERLAY = 1
	else:
	    OVERLAY = 0
	filesystem = storage.defaultFSType
	backup_size = Size("0GiB")
	swap_size = Size("0GiB")
	mem_size = util.total_memory()
	OVER_SIZE = Size("0GiB")
	for d in storage.disks:
	    if d.name in storage.ksdata.clearpart.drives[0]:
		partition_layout = []
		BACKUP = ["backup","ext4","30GiB"]
		if d.size < Size("93GiB"): #100G
		    if OVERLAY == 0 :
			ROOT = ["root","ext3","1"]
			SWAP = ["swap","swap","4GiB"]
			swap_size = Size("4GiB")
			backup_size = Size("0GiB")
                        OVER_SIZE = Size("0GiB")
			partition_layout.append(ROOT)
			partition_layout.append(SWAP)
		    else:
		        #if OVERLAY == 1 :
			if d.size < Size("26GiB"): #16G
                    	    ROOT = ["root","ext3","1"]
                    	    SWAP = ["swap","swap","4GiB"]
                    	    swap_size = Size("4GiB")
			    backup_size = Size("0GiB")
                            OVER_SIZE = Size("0GiB")
                     	    partition_layout.append(ROOT)
                    	    partition_layout.append(SWAP)
			elif d.size < Size("50GiB"): #32G
                            ROOT = ["root","ext3","20GiB"]
                            SWAP = ["swap","swap","4GiB"]
			    OVER = ["overlay","ext4","1"]
                            swap_size = Size("4GiB")
			    backup_size = Size("0GiB")
                            OVER_SIZE = Size("0GiB")
			    partition_layout.append(ROOT)
                            partition_layout.append(SWAP)
			    partition_layout.append(OVER)         
			else: #80G
                            ROOT = ["root","ext3","20GiB"]
                            SWAP = ["swap","swap","4GiB"]
			    WORK = ["work","ext4","4GiB"]
			    DATA = ["data","ext4","1"]
                            OVER = ["overlay","ext4","20GiB"]
                            swap_size = Size("4GiB")
			    backup_size = Size("0GiB")
                            OVER_SIZE = Size("20GiB")  #20 + 4
			    partition_layout.append(ROOT)
                            partition_layout.append(SWAP)
			    partition_layout.append(WORK)
                            partition_layout.append(DATA)
                            partition_layout.append(OVER)

		elif d.size < Size("220GiB"): #240G
		    swap_size = Size(mem_size)
		    backup_size = Size("30GiB")
		    ROOT = ["root","ext3","0.66"]
		    SWAP = ["swap","swap",str(swap_size)]
		    DATA = ["data","ext4","1"]
		    partition_layout.append(ROOT)
		    partition_layout.append(SWAP)
		    partition_layout.append(BACKUP)
		    partition_layout.append(DATA)
		    if OVERLAY == 1 :			   
			OVER = ["overlay","ext4","20GiB"]
		        partition_layout.append(OVER)
			OVER_SIZE = Size("20GiB")



		elif d.size < Size("465GiB"): #500G
		    swap_size = Size(mem_size)
		    backup_size = Size("30GiB")
		    ROOT = ["root","ext3","0.66"]
		    SWAP = ["swap","swap",str(swap_size)]
		    DATA = ["data","ext4","1"]
		    partition_layout.append(ROOT)
		    partition_layout.append(SWAP)
		    partition_layout.append(BACKUP)
		    partition_layout.append(DATA)
		    if OVERLAY == 1 :			   
			OVER = ["overlay","ext4","30GiB"]
		        partition_layout.append(OVER)
			OVER_SIZE = Size("30GiB")

		else:
		    swap_size_tmp = str(round((float(str(mem_size).split()[0]) * 1.2),2)) + str(mem_size).split()[1]
		    swap_size = Size(swap_size_tmp)
		    backup_size = Size("30GiB")
		    ROOT = ["root","ext3","0.33"]
		    SWAP = ["swap","swap",str(swap_size)]
		    DATA = ["data","ext4","1"]
		    partition_layout.append(ROOT)
		    partition_layout.append(SWAP)
		    partition_layout.append(BACKUP)
		    partition_layout.append(DATA)
		    if OVERLAY == 1 :			   
			OVER = ["overlay","ext4","30GiB"]
		        partition_layout.append(OVER)
			OVER_SIZE = Size("30GiB")
		print partition_layout
		log.debug("partition_layout[]:\n%s",partition_layout)

	autorequests = []
	disk_size = Size("0GiB")
	if len(storage.ksdata.clearpart.drives) < 1:
	    autorequests.append(PartSpec(mountpoint="/", fstype=filesystem,
                    size=Size("30GiB"), grow=False, lv=False))
	    log.debug("this is autopart for left space,only create 30 GiB root partition")
	    storage.autoPartitionRequests = autorequests
	    return
	else:
	    disk_name = storage.ksdata.clearpart.drives[0]
	    for i in storage.devices:
		if i.name == disk_name:
		    disk_size = i.size
		    break
	    if disk_size < Size("1GiB"):
		log.debug("disk size %s is too small :%d",disk_name,disk_size)
		return 1

	used_size = swap_size + backup_size + OVER_SIZE
	for i in partition_layout:
	    use_all_space = False
	    if i[1] == "NONE":
		defaulttype = filesystem
	    else:
		defaulttype = i[1]
	    pre_size = i[2]
	    if i[2].startswith("0."):
		pre_size = str(round(float(str(disk_size - used_size).split()[0]) * float(i[2]),2)) + "GiB"
	    elif i[2].endswith("iB"):
		pass
	    elif i[2] == "1":
		use_all_space = True
		pre_size = "1GiB"
	    else:
		print "-------OOC"

	    if i[0] == "root":
		autorequests.append(PartSpec(mountpoint="/",
				    fstype=defaulttype, size=Size(pre_size),
				    grow=use_all_space,
				    btr=True, lv=True, thin=True, encrypted=True))
		log.debug("mountpoint=/ fstype=%s size=Size\(%s\), grow=%s, lv=True",defaulttype,pre_size,use_all_space)
	    elif i[0] == "swap":
		autorequests.append(PartSpec(mountpoint="swap",
					fstype=defaulttype, size=Size(pre_size),
					grow=False,
				    lv=True, encrypted=True))
		log.debug("mountpoint=swap fstype=%s size=Size\(%s\), grow=False, lv=True",defaulttype,pre_size)
	    elif i[0] == "backup":
		autorequests.append(PartSpec(mountpoint="/srv",
				    fstype=defaulttype, size=Size(pre_size),
				    grow=False,
				    btr=True, lv=True, thin=True, encrypted=True))
		log.debug("mountpoint=/srv fstype=%s size=Size\(%s\), grow=False, lv=True",defaulttype,pre_size)
	    elif i[0] == "data":
		autorequests.append(PartSpec(mountpoint="/DATA",
				    fstype=defaulttype, size=Size(pre_size),
				    grow=use_all_space,
				    btr=True, lv=True, thin=True, encrypted=True))
		log.debug("mountpoint=/DATA fstype=%s size=Size\(%s\), grow=%s, lv=True",defaulttype,pre_size,use_all_space)
	    elif i[0] == "work":
		autorequests.append(PartSpec(mountpoint="/WORK",
				    fstype=defaulttype, size=Size(pre_size),
				    grow=use_all_space,
				    btr=True, lv=True, thin=True, encrypted=True))
		log.debug("mountpoint=/WORK fstype=%s size=Size\(%s\), grow=%s, lv=True",defaulttype,pre_size,use_all_space)
	    elif i[0] == "overlay":
		autorequests.append(PartSpec(mountpoint="/overlay",
				    fstype=defaulttype, size=Size(pre_size),
				    grow=use_all_space,
				    btr=True, lv=True, thin=True, encrypted=True))
		log.debug("mountpoint=/overlay fstype=ext4 size=Size\(%s\), grow=False, lv=True",Size(pre_size))
	    else:
		autorequests.append(PartSpec(mountpoint=i[0],
				    fstype=defaulttype, size=Size(pre_size),
				    grow=use_all_space,
				    btr=True, lv=True, thin=True, encrypted=True))
		log.debug("mountpoint=%s fstype=%s size=Size\(%s\), grow=%s, lv=True",i[0],defaulttype,pre_size,use_all_space)

	storage.autoPartitionRequests = autorequests

	'''
        autorequests = [PartSpec(mountpoint="/", fstype=storage.defaultFSType,
                                 size=Size("1GiB"),
                                 maxSize=Size("50GiB"),
                                 grow=True,
                                 btr=True, lv=True, thin=True, encrypted=True),
                        PartSpec(mountpoint="/backup",
                                 fstype=storage.defaultFSType,
                                 size=Size("1GiB"),
                                 maxSize=Size("30GiB"),
                                 grow=True,
                                 btr=True, lv=True, thin=True, encrypted=True)]

        bootreqs = platform.setDefaultPartitioning()
        if bootreqs:
            autorequests.extend(bootreqs)


        disk_space = getAvailableDiskSpace(storage)
        swp = swap.swapSuggestion(disk_space=disk_space)
        autorequests.append(PartSpec(fstype="swap", size=swp, grow=False,
                                     lv=True, encrypted=True))

        for autoreq in autorequests:
            if autoreq.fstype is None:
                if autoreq.mountpoint == "/boot":
                    autoreq.fstype = storage.defaultBootFSType
                else:
                    autoreq.fstype = storage.defaultFSType

        storage.autoPartitionRequests = autorequests
	'''

    def configure(self, anaconda):
        anaconda.bootloader.timeout = self.bootloaderTimeoutDefault
        anaconda.bootloader.boot_args.update(self.bootloaderExtraArgs)

    # sets default ONBOOT values and updates ksdata accordingly
    def setNetworkOnbootDefault(self, ksdata):
        pass

    def __init__(self):
        pass

allClasses = []
allClasses_hidden = []

# returns ( className, classObject ) tuples
def availableClasses(showHidden=False):
    global allClasses
    global allClasses_hidden

    def _ordering(first, second):
        ((name1, _), priority1) = first
        ((name2, _), priority2) = second

        if priority1 < priority2:
            return -1
        elif priority1 > priority2:
            return 1

        if name1 < name2:
            return -1
        elif name1 > name2:
            return 1

        return 0

    if not showHidden:
        if allClasses:
            return allClasses
    else:
        if allClasses_hidden:
            return allClasses_hidden

    path = []

    env_path = []
    if "ANACONDA_INSTALL_CLASSES" in os.environ:
        env_path += os.environ["ANACONDA_INSTALL_CLASSES"].split(":")

    for d in env_path + ["installclasses",
              "/tmp/updates/pyanaconda/installclasses",
              "/tmp/product/pyanaconda/installclasses",
              "%s/pyanaconda/installclasses" % get_python_lib(plat_specific=1) ]:
        if os.access(d, os.R_OK):
            path.append(d)

    # append the location of installclasses to the python path so we
    # can import them
    sys.path = path + sys.path

    files = []
    for p in reversed(path):
        files += os.listdir(p)

    done = {}
    lst = []
    for fileName in files:
        if fileName[0] == '.':
            continue
        if len (fileName) < 4:
            continue
        if fileName[-3:] != ".py" and fileName[-4:-1] != ".py":
            continue
        mainName = fileName.split(".")[0]
        if mainName in done:
            continue
        done[mainName] = 1

        try:
            found = imputil.imp.find_module(mainName)
        except ImportError:
            log.warning ("module import of %s failed: %s", mainName, sys.exc_type)
            continue

        try:
            loaded = imputil.imp.load_module(mainName, found[0], found[1], found[2])

            for (_key, obj) in loaded.__dict__.items():
                # If it's got these two methods, it's an InstallClass.
                if hasattr(obj, "setDefaultPartitioning") and hasattr(obj, "setPackageSelection"):
                    sortOrder = getattr(obj, "sortPriority", 0)
                    if not obj.hidden or showHidden:
                        lst.append(((obj.name, obj), sortOrder))
        except (ImportError, AttributeError):
            log.warning ("module import of %s failed: %s", mainName, sys.exc_type)

    lst.sort(_ordering)
    for (item, _) in lst:
        if showHidden:
            allClasses_hidden += [item]
        else:
            allClasses += [item]

    if showHidden:
        return allClasses_hidden
    else:
        return allClasses

def getBaseInstallClass():
    # figure out what installclass we should base on.
    allavail = availableClasses(showHidden=True)
    avail = availableClasses(showHidden=False)

    if len(avail) == 1:
        (cname, cobject) = avail[0]
        log.info("using only installclass %s", cname)
    elif len(allavail) == 1:
        (cname, cobject) = allavail[0]
        log.info("using only installclass %s", cname)

    # Use the highest priority install class if more than one found.
    elif len(avail) > 1:
        (cname, cobject) = avail.pop()
        log.info('%s is the highest priority installclass, using it', cname)
    elif len(allavail) > 1:
        (cname, cobject) = allavail.pop()
        log.info('%s is the highest priority installclass, using it', cname)

    # Default to the base installclass if nothing else is found.
    else:
        raise RuntimeError("Unable to find an install class to use!!!")

    return cobject

baseclass = getBaseInstallClass()

# we need to be able to differentiate between this and custom
class DefaultInstall(baseclass):
    def __init__(self):
        baseclass.__init__(self)
